//To compile the project you need the extended version of the CADImport .NET library
//that allows to access CAD entities directly.  You can get this version of the 
//library free of charge by emailing your request to: info@cadsofttools.com
using System;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Globalization;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using CADImport;
using CADImport.CADImportForms;
using CADImport.HPGL2;
using CADImport.RasterImage;
using CADImportNetDemos.CADImportDemo.TXTImport;
using CADImportNetDemos.CADImportDemo.XMLImport;

namespace CADImportNetDemos.CADImportDemo
{
	#region Help
	/// <summary>
	/// Represents the main application form in which CAD image can be viewed.
	/// </summary>
	#endregion Help
	public class MainForm : System.Windows.Forms.Form
	{			
		private CADImport.CADImportForms.AboutForm aboutFrm;
		#region protect
#if (protect && floatprotect)
		private CADImport.CADImportForms.FloatLicForm floatLicFrm;
#endif
#if protect
		private CADImport.CADImportForms.RegForm regFrm;
#endif
		#endregion		
		
		private float sc;				
		private float prev_sc;				
		private int cX, cY;
		private bool detRMouseDown;		
		private System.Drawing.PointF old_Pos;				
		private System.Drawing.PointF pos;		
		private System.Drawing.SizeF visibleArea;				
		private CADImport.CADImage cadImage;		
		private CADImport.FaceModule.CADPictureBox cadPictBox;	
		private CADImportNetDemos.CADImportDemo.XMLImport.XMLImporter xmlImporter;
		private CADImportNetDemos.CADImportDemo.TXTImport.TXTImporter txtImporter;	
				
		private System.Windows.Forms.StatusBar stBar;
		private System.Windows.Forms.StatusBarPanel sbOpen;
		private System.Windows.Forms.StatusBarPanel sbScale;
		private System.Windows.Forms.StatusBarPanel sbCoord;
		private System.Windows.Forms.Panel pnlLayouts;		
		private System.Windows.Forms.MenuItem miFloatLic;
		private System.Windows.Forms.MenuItem miOpenFile;
		private System.Windows.Forms.MenuItem miSaveFile;
		private System.Windows.Forms.MenuItem miExit;
		private System.Windows.Forms.MenuItem miScale;
		private System.Windows.Forms.MenuItem miZoom10;
		private System.Windows.Forms.MenuItem miZoom25;
		private System.Windows.Forms.MenuItem miZoom50;
		private System.Windows.Forms.MenuItem miZoom100;
		private System.Windows.Forms.MenuItem miZoom200;
		private System.Windows.Forms.MenuItem miZoom400;
		private System.Windows.Forms.MenuItem miZoom800;
		private System.Windows.Forms.Button btnSave;
		private System.Windows.Forms.Button btnLoad;
		private System.Windows.Forms.MenuItem miFile;
		private System.Windows.Forms.MenuItem miSep;
		private System.Windows.Forms.MenuItem miView;
		private System.Windows.Forms.MenuItem miHelp;
		private System.Windows.Forms.MenuItem miRegister;
		private System.Windows.Forms.MenuItem miAbout;
		private System.Windows.Forms.OpenFileDialog dlgOpenFile;
		private System.Windows.Forms.SaveFileDialog dlgSaveImg;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.SaveFileDialog dlgSaveEnt;		

		#region Help
		/// <summary>
		/// A previous position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>
		#endregion Help
		public PointF PreviousPosition
		{
			get
			{
				return this.old_Pos;
			}
			set
			{
				this.old_Pos = value;
			}
		}			
	
		#region Help
#if RUSHELP
		/// <summary>
		///      
		/// </summary>
#else		
		/// <summary>
		/// A position of the current CAD image in relation to the top-left corner of the viewer.
		/// </summary>		
#endif
		#endregion
		public PointF Position
		{
			get
			{
				return this.pos;
			}
			set
			{
				this.pos = value;
			}
		}

		#region Help
		/// <summary>
		/// An X coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentXClickPosition
		{
			get
			{
				return this.cX;
			}
		}
		#region Help
		/// <summary>
		/// A Y coordinate of the last mouse click position on the screen.
		/// </summary>
		#endregion Help
		public int CurrentYClickPosition
		{
			get
			{
				return this.cY;
			}
		}
		
		#region Help
		/// <summary>
		/// A CAD image displayed in the main application form.
		/// </summary>
		#endregion Help		
		public CADImage Image
		{
			get
			{
				return cadImage;
			}
			set
			{
				this.cadImage = value;
			}
		}

		#region Help
		/// <summary>
		/// A scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImageScale
		{
			get
			{
				return this.sc;
			}
			set
			{
				this.sc = value;
			}
		}

		#region Help
		/// <summary>
		/// A previous value of the scale in which the current CAD image is displayed in the viewer.
		/// </summary>
		#endregion Help
		public float ImagePreviousScale
		{
			get
			{
				return this.prev_sc;
			}
			set
			{
				this.prev_sc = value;
			}
		}
		
		#region Help
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
		/// <summary>
		/// Gets or sets base dimensions of the image drawing area
		/// </summary>
#endif
		#endregion
		public SizeF VisibleAreaSize
		{
			get
			{
				return this.visibleArea;
			}
			set
			{
				this.visibleArea = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         X
		/// </summary>
#else			
		/// <summary>
		/// Gets or sets a position of the drawing image by X axis
		/// </summary>
#endif
		#endregion
		public float LeftImagePosition
		{
			get
			{
				return this.pos.X;
			}
			set
			{
				this.pos.X = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///         Y
		/// </summary>
#else
		/// <summary>
		/// Gets or sets a position of the drawing image by Y axis
		/// </summary>
#endif
		#endregion
		public float TopImagePosition
		{
			get
			{
				return this.pos.Y;
			}
			set
			{
				this.pos.Y = value;
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public RectangleF ImageRectangleF
		{
			get
			{
				return new RectangleF(this.LeftImagePosition, this.TopImagePosition, 
									  VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		#region Help
#if RUSHELP
		/// <summary>
		///       
		/// </summary>
#else
		/// <summary>
		/// Gets the current size and position of the drawing image
		/// </summary>
#endif
		#endregion
		public DRect ImageDRect
		{
			get
			{
				return new DRect(this.LeftImagePosition, this.TopImagePosition, 
								 VisibleAreaSize.Width * ImageScale, VisibleAreaSize.Height * ImageScale);								
			}
		}

		private string RealScale
		{
			get
			{
				if(cadImage != null)
					return string.Format("{0,2:F}%", (((this.VisibleAreaSize.Width * ImageScale) / this.cadImage.AbsWidth) * cadImage.MMToPixelX) * 100);
				else 
					return string.Format("{0}%", ImageScale);
			}
		}

		private void InitParams()
		{			
			aboutFrm = new AboutForm();			
			xmlImporter = new XMLImporter(); 		
			txtImporter = new TXTImporter();					
			#region protect
#if (protect && floatprotect)
			floatLicFrm = new FloatLicForm();
#endif
#if protect
			regFrm = new RegForm();
#endif
			#endregion protect
		}

		private void InitExtParam()
		{
			this.cadPictBox.MouseWheel += new MouseEventHandler(CADPictBoxMouseWheel);
			this.cadPictBox.ScrollEvent += new CADImport.FaceModule.ScrollEventHandlerExt(CADPictBoxScroll);			
			#region protect
#if (protect && floatprotect)		
			floatLicFrm.Change += new System.EventHandler(InvalidateImage);
			#else
			this.miFloatLic.Enabled = false;
#endif
			#endregion protect
			old_Pos = new PointF();
			sc = 1.0f;
			prev_sc = 1.0f;
		}
		
		#region Help
		/// <summary>
		/// Initializes a new instance of the <see cref="MainForm">MainForm</see> class.
		/// </summary>
		#endregion Help
		public MainForm()
		{
			InitParams();		
			InitializeComponent();
			InitExtParam();
		}						

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(MainForm));
			this.dlgOpenFile = new System.Windows.Forms.OpenFileDialog();
			this.dlgSaveImg = new System.Windows.Forms.SaveFileDialog();
			this.stBar = new System.Windows.Forms.StatusBar();
			this.sbOpen = new System.Windows.Forms.StatusBarPanel();
			this.sbScale = new System.Windows.Forms.StatusBarPanel();
			this.sbCoord = new System.Windows.Forms.StatusBarPanel();
			this.mainMenu = new System.Windows.Forms.MainMenu();
			this.miFile = new System.Windows.Forms.MenuItem();
			this.miOpenFile = new System.Windows.Forms.MenuItem();
			this.miSaveFile = new System.Windows.Forms.MenuItem();
			this.miSep = new System.Windows.Forms.MenuItem();
			this.miExit = new System.Windows.Forms.MenuItem();
			this.miView = new System.Windows.Forms.MenuItem();
			this.miScale = new System.Windows.Forms.MenuItem();
			this.miZoom10 = new System.Windows.Forms.MenuItem();
			this.miZoom25 = new System.Windows.Forms.MenuItem();
			this.miZoom50 = new System.Windows.Forms.MenuItem();
			this.miZoom100 = new System.Windows.Forms.MenuItem();
			this.miZoom200 = new System.Windows.Forms.MenuItem();
			this.miZoom400 = new System.Windows.Forms.MenuItem();
			this.miZoom800 = new System.Windows.Forms.MenuItem();
			this.miHelp = new System.Windows.Forms.MenuItem();
			this.miRegister = new System.Windows.Forms.MenuItem();
			this.miFloatLic = new System.Windows.Forms.MenuItem();
			this.miAbout = new System.Windows.Forms.MenuItem();
			this.pnlLayouts = new System.Windows.Forms.Panel();
			this.btnSave = new System.Windows.Forms.Button();
			this.btnLoad = new System.Windows.Forms.Button();
			this.cadPictBox = new CADImport.FaceModule.CADPictureBox();
			this.dlgSaveEnt = new System.Windows.Forms.SaveFileDialog();
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).BeginInit();
			this.pnlLayouts.SuspendLayout();
			this.SuspendLayout();
			// 
			// dlgOpenFile
			// 
			this.dlgOpenFile.DefaultExt = "*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.p" +
				"rn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf" +
				"; *.wmf; *.cgm";
			this.dlgOpenFile.Filter = @"All(*.dxf,*.dwg,*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl,*.cgm, *.bmp,*.jpg, *.jpeg,*.tiff, *.gif, *.ico, *.cur, *.png, *.emf, *.wmf, *.gbr)|*.bmp;*.jpg;*.jpeg;*.tiff; *.gif; *.ico;*.dxf;*.dwg;*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl;*.cur;*.png; *.emf; *.wmf; *.cgm;*.gbr|CAD files(*.dxf, *.dwg)|*.dxf;*.dwg|All raster files (*.bmp,*.jpg, *.jpeg,*.tiff,*.tif, *.gif, *.ico, *.cur, *.png)|*.bmp;*.jpg;*.jpeg;*.tiff;*.tif; *.gif;*.ico;*.cur; *.png|Metafiles ( *.emf, *.wmf)| *.emf; *.wmf|DXF files(*.dxf)|*.dxf|DWG files(*.dwg)|*.dwg|BMP(*.bmp)|*.bmp|JPG(*.jpg, *.jpeg)| *.jpg; *.jpeg|TIFF(*.tiff, *.tif)|*.tiff;*.tif|GIF(*.gif)|*.gif|ICO(*.ico)|*.ico|PNG(*.png)|*.png|Cursors(*.cur)|*.cur|HPGL(*.hpg,*.plt,*.rtl, *.spl, *.prn, *.gl2,*.hpgl2,*.hpgl,*.hp2,*.hp1, *.hp, *.plo, *.hg,*.hgl)|*.hpg;*.plt;*.rtl; *.spl; *.prn; *.gl2;*.hpgl2;*.hpgl;*.hp2;*.hp1; *.hp; *.plo; *.hg;*.hgl|CGM (*.cgm)|*.cgm|(Gerber)*.gbr|*.gbr";
			this.dlgOpenFile.ShowHelp = true;
			// 
			// dlgSaveImg
			// 
			this.dlgSaveImg.Filter = "*.bmp|*.bmp|*.jpg|*.jpg|*.tiff|*.tiff|*.gif|*.gif|*.emf|*.emf";
			this.dlgSaveImg.RestoreDirectory = true;
			// 
			// stBar
			// 
			this.stBar.Font = new System.Drawing.Font("Microsoft Sans Serif", 8F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(204)));
			this.stBar.Location = new System.Drawing.Point(0, 300);
			this.stBar.Name = "stBar";
			this.stBar.Panels.AddRange(new System.Windows.Forms.StatusBarPanel[] {
																					 this.sbOpen,
																					 this.sbScale,
																					 this.sbCoord});
			this.stBar.ShowPanels = true;
			this.stBar.Size = new System.Drawing.Size(687, 16);
			this.stBar.TabIndex = 9;
			// 
			// sbOpen
			// 
			this.sbOpen.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbOpen.MinWidth = 100;
			this.sbOpen.ToolTipText = "File Name";
			// 
			// sbScale
			// 
			this.sbScale.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Spring;
			this.sbScale.ToolTipText = "Scale";
			this.sbScale.Width = 520;
			// 
			// sbCoord
			// 
			this.sbCoord.Alignment = System.Windows.Forms.HorizontalAlignment.Right;
			this.sbCoord.AutoSize = System.Windows.Forms.StatusBarPanelAutoSize.Contents;
			this.sbCoord.Text = "0 - 0 - 0";
			this.sbCoord.Width = 51;
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.miFile,
																					 this.miView,
																					 this.miHelp});
			// 
			// miFile
			// 
			this.miFile.Index = 0;
			this.miFile.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miOpenFile,
																				   this.miSaveFile,
																				   this.miSep,
																				   this.miExit});
			this.miFile.Text = "&File";
			// 
			// miOpenFile
			// 
			this.miOpenFile.Index = 0;
			this.miOpenFile.Text = "&Open File...";
			this.miOpenFile.Click += new System.EventHandler(this.miOpenFile_Click);
			// 
			// miSaveFile
			// 
			this.miSaveFile.Enabled = false;
			this.miSaveFile.Index = 1;
			this.miSaveFile.Text = "Save File As...";
			this.miSaveFile.Click += new System.EventHandler(this.miSaveFile_Click);
			// 
			// miSep
			// 
			this.miSep.Index = 2;
			this.miSep.Text = "-";
			// 
			// miExit
			// 
			this.miExit.Index = 3;
			this.miExit.Text = "&Close";
			this.miExit.Click += new System.EventHandler(this.miExit_Click);
			// 
			// miView
			// 
			this.miView.Index = 1;
			this.miView.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miScale});
			this.miView.Text = "&View";
			// 
			// miScale
			// 
			this.miScale.Enabled = false;
			this.miScale.Index = 0;
			this.miScale.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					this.miZoom10,
																					this.miZoom25,
																					this.miZoom50,
																					this.miZoom100,
																					this.miZoom200,
																					this.miZoom400,
																					this.miZoom800});
			this.miScale.Text = "&Scale";
			// 
			// miZoom10
			// 
			this.miZoom10.Index = 0;
			this.miZoom10.Text = "10%";
			this.miZoom10.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom25
			// 
			this.miZoom25.Index = 1;
			this.miZoom25.Text = "25%";
			this.miZoom25.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom50
			// 
			this.miZoom50.Index = 2;
			this.miZoom50.Text = "50%";
			this.miZoom50.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom100
			// 
			this.miZoom100.Index = 3;
			this.miZoom100.Text = "100%";
			this.miZoom100.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom200
			// 
			this.miZoom200.Index = 4;
			this.miZoom200.Text = "200%";
			this.miZoom200.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom400
			// 
			this.miZoom400.Index = 5;
			this.miZoom400.Text = "400%";
			this.miZoom400.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miZoom800
			// 
			this.miZoom800.Index = 6;
			this.miZoom800.Text = "800%";
			this.miZoom800.Click += new System.EventHandler(this.miZoom_Click);
			// 
			// miHelp
			// 
			this.miHelp.Index = 2;
			this.miHelp.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				   this.miRegister,
																				   this.miFloatLic,
																				   this.miAbout});
			this.miHelp.Text = "&?";
			// 
			// miRegister
			// 
			this.miRegister.Index = 0;
			this.miRegister.Text = "Register";
			this.miRegister.Click += new System.EventHandler(this.miRegister_Click);
			// 
			// miFloatLic
			// 
			this.miFloatLic.Index = 1;
			this.miFloatLic.Text = "Floating License Registration...";
			this.miFloatLic.Click += new System.EventHandler(this.miFloatLic_Click);
			// 
			// miAbout
			// 
			this.miAbout.Index = 2;
			this.miAbout.Text = "&About";
			this.miAbout.Click += new System.EventHandler(this.miAbout_Click);
			// 
			// pnlLayouts
			// 
			this.pnlLayouts.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.pnlLayouts.Controls.Add(this.btnSave);
			this.pnlLayouts.Controls.Add(this.btnLoad);
			this.pnlLayouts.Dock = System.Windows.Forms.DockStyle.Top;
			this.pnlLayouts.Location = new System.Drawing.Point(0, 0);
			this.pnlLayouts.Name = "pnlLayouts";
			this.pnlLayouts.Size = new System.Drawing.Size(687, 32);
			this.pnlLayouts.TabIndex = 11;
			// 
			// btnSave
			// 
			this.btnSave.Enabled = false;
			this.btnSave.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnSave.Location = new System.Drawing.Point(96, 4);
			this.btnSave.Name = "btnSave";
			this.btnSave.TabIndex = 1;
			this.btnSave.Text = "Save";
			this.btnSave.Click += new System.EventHandler(this.miSaveFile_Click);
			// 
			// btnLoad
			// 
			this.btnLoad.FlatStyle = System.Windows.Forms.FlatStyle.System;
			this.btnLoad.Location = new System.Drawing.Point(9, 3);
			this.btnLoad.Name = "btnLoad";
			this.btnLoad.TabIndex = 0;
			this.btnLoad.Text = "Load";
			this.btnLoad.Click += new System.EventHandler(this.miOpenFile_Click);
			// 
			// cadPictBox
			// 
			this.cadPictBox.BackColor = System.Drawing.Color.White;
			this.cadPictBox.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.cadPictBox.Cursor = System.Windows.Forms.Cursors.Default;
			this.cadPictBox.Dock = System.Windows.Forms.DockStyle.Fill;
			this.cadPictBox.DoubleBuffering = true;
			this.cadPictBox.Location = new System.Drawing.Point(0, 32);
			this.cadPictBox.Name = "cadPictBox";
			this.cadPictBox.Position = new System.Drawing.Point(0, 0);
			this.cadPictBox.Size = new System.Drawing.Size(687, 268);
			this.cadPictBox.TabIndex = 8;
			this.cadPictBox.TabStop = false;
			this.cadPictBox.VirtualSize = new System.Drawing.Size(0, 0);
			this.cadPictBox.MouseUp += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseUp);
			this.cadPictBox.Paint += new System.Windows.Forms.PaintEventHandler(this.cadPictBox_Paint);
			this.cadPictBox.MouseMove += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseMove);
			this.cadPictBox.MouseDown += new System.Windows.Forms.MouseEventHandler(this.cadPictBox_MouseDown);
			// 
			// dlgSaveEnt
			// 
			this.dlgSaveEnt.DefaultExt = "*.xml";
			this.dlgSaveEnt.Filter = "*.xml|*.xml|*.txt|*.txt";
			// 
			// MainForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.SystemColors.Control;
			this.ClientSize = new System.Drawing.Size(687, 316);
			this.Controls.Add(this.cadPictBox);
			this.Controls.Add(this.pnlLayouts);
			this.Controls.Add(this.stBar);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.Menu = this.mainMenu;
			this.Name = "MainForm";
			this.RightToLeft = System.Windows.Forms.RightToLeft.No;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "CADImport.NET - Import";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Closing += new System.ComponentModel.CancelEventHandler(this.MainForm_Closing);
			this.Load += new System.EventHandler(this.MainForm_Load);
			this.Deactivate += new System.EventHandler(this.MainForm_Deactivate);
			((System.ComponentModel.ISupportInitialize)(this.sbOpen)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbScale)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.sbCoord)).EndInit();
			this.pnlLayouts.ResumeLayout(false);
			this.ResumeLayout(false);

		}
		#endregion
		
		[STAThread]
		static void Main() 
		{
			Application.EnableVisualStyles();
			Application.DoEvents();			
			Application.ThreadException += new System.Threading.ThreadExceptionEventHandler(DoExceptionWorking);
			Application.Run(new MainForm());			
		}

		private static void DoExceptionWorking(object sender, System.Threading.ThreadExceptionEventArgs args)
		{				
			CADConst.SendExceptionReport(args.Exception);
		}

		private void CADPictBoxScroll(object sender, CADImport.FaceModule.ScrollEventArgsExt e)
		{			
			LeftImagePosition = -this.cadPictBox.Position.X;
			TopImagePosition = -this.cadPictBox.Position.Y;
			this.cadPictBox.Invalidate();
		}

		private void Zoom(float i)
		{
			if(cadImage == null) 
				return;
			sc = sc * i;
			if(sc < 0.005f) 
				sc = 0.005f;	
			cadPictBox.Invalidate();			
		}
		
		private void Shift()
		{
			LeftImagePosition = PreviousPosition.X - (PreviousPosition.X - LeftImagePosition) * ImageScale / ImagePreviousScale;
			TopImagePosition = PreviousPosition.Y - (PreviousPosition.Y - TopImagePosition) * ImageScale / ImagePreviousScale;
			ImagePreviousScale = ImageScale;		
		}	

		private void CADPictBoxMouseWheel(object sender, System.Windows.Forms.MouseEventArgs e)
		{			
			if(e.Delta < 0)	
				Zoom(0.7f);
			else 
				Zoom(1.3f);
			this.Shift();
			SetPictureBoxPosition(Position);
		}		

		private void InvalidateImage(object sender, System.EventArgs e)
		{
			this.cadPictBox.Invalidate();
		}

		private void miOpenFile_Click(object sender, System.EventArgs e)
		{
			this.ShowLoadFileDialog();	
			this.ChangeControlState();			
		}

		#region Help
		/// <summary>
		/// Enables or disables buttons of the toolbar and menu items of the main menu.
		/// </summary>
		/// <param name="aVal"><b>true</b> if to enable the buttons and menu items; <b>false</b> if to disable them.</param>
		/// <remarks>The buttons and menu items are enabled after loading a CAD file and become disabled after closing the file.</remarks>
		#endregion Help
		public void EnableButton(bool aVal)
		{			
			miScale.Enabled = aVal;
			miSaveFile.Enabled = aVal;					
			btnSave.Enabled = aVal;
		}

		private void ChangeControlState()
		{
			bool detNullImg = cadImage != null;
			EnableButton(detNullImg);			
		}

		#region Help
#if RUSHELP
		/// <summary>
		///        
		/// </summary>
#else
		/// <summary>
		/// Invokes open file dialog and loads specified file
		/// </summary>
#endif
		#endregion
		public void ShowLoadFileDialog()
		{			
			this.cadPictBox.Enabled = false;
			if(dlgOpenFile.ShowDialog() != DialogResult.OK) 
			{
				cadPictBox.Visible = true;	
				this.cadPictBox.Enabled = true;
				return;
			}
			this.cadPictBox.Enabled = true;
			LoadFile(dlgOpenFile.FileName);			
		}

		#region Help
		/// <summary>
		/// Loads a CAD file.
		/// </summary>
		#endregion Help
		public void LoadFile(string fileName)
		{									
			if(fileName != null)
			{
				if(cadImage != null)
				{	
					cadImage.Dispose();
					cadImage = null;					
				}
				this.Cursor = Cursors.WaitCursor;
				stBar.Panels[0].Text = ApplicationConstants.loadfilestr;
				ImageScale = 1.0f;
				ImagePreviousScale = 1.0f;
				Position = new PointF();				
				this.cadImage = CADImage.CreateImageByExtension(fileName);																
				if(this.cadImage is CADRasterImage)
					(this.cadImage as CADRasterImage).Control = this.cadPictBox;
			} 
			if(this.cadImage != null)
			{									
				if(CADConst.IsWebPath(fileName))
					this.cadImage.LoadFromWeb(fileName);
				else
					cadImage.LoadFromFile(fileName);								
			}
			stBar.Panels[0].Text = Path.GetFileName(fileName) + ApplicationConstants.sepstr;			
			CADImage.LastLoadedFilePath = Path.GetDirectoryName(fileName); 
			SetCADImageOptions();
		}

		#region Help
#if RUSHELP
		/// <summary>
		///     <see cref="MainForm.Image"/>CADImage    
		/// </summary>
#else
		/// <summary>
		/// Sets current properties for <see cref="MainForm.Image"/>CADImage for loading file
		/// </summary>
#endif
		#endregion
		public void SetCADImageOptions()
		{			
			cadImage.UseWinEllipse = false;		
			cadImage.IsShowLineWeight = false;						
			this.cadImage.UseDoubleBuffering = true;									
			this.Cursor = Cursors.Default;									
			ObjEntity.cadImage = cadImage;									
			xmlImporter.Image = this.cadImage;
			txtImporter.Image = this.cadImage;
			DoResize();			
			SetPictureBoxPosition(Position);					
		}

		#region Help
#if RUSHELP
		/// <summary>
		///      <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz"> </param>
#else
		/// <summary>
		/// Sets scroll region for <see cref="CADImport.FaceModule.CADPictureBox">CADPictureBox</see>
		/// </summary>
		/// <param name="sz">Size to set</param>
#endif
		#endregion
		public void SetSizePictureBox(Size sz)
		{		
			this.cadPictBox.SetVirtualSizeNoInvalidate(sz);			
		}

		private void SetPictureBoxPosition(PointF value)
		{
			int w1, h1;
			if(value.X > 0)
				w1 = 0;
			else
				w1 = (int)Math.Abs(value.X);
			if(w1 > this.cadPictBox.VirtualSize.Width)
				w1 = this.cadPictBox.VirtualSize.Width;
			if(value.Y > 0)
				h1 = 0;
			else
				h1 = (int)Math.Abs(value.Y);
			if(h1 > this.cadPictBox.VirtualSize.Height)
				h1 = this.cadPictBox.VirtualSize.Height;
			this.cadPictBox.SetPositionNoInvalidate(new Point(w1, h1));
		}

		private void NotSupportedInCurrentVersion()
		{
			this.Cursor = Cursors.Default;
			cadPictBox.Visible = true;
			stBar.Panels[0].Text = ApplicationConstants.notsupportedstr;
			MessageBox.Show(ApplicationConstants.notsupportedextstr, ApplicationConstants.appnamestr);
			return;
		}				

		private void cadPictBox_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
		{
			if(cadImage == null) 
				return;			
			DrawCADImage(e.Graphics);
			stBar.Panels[1].Text = RealScale;										
		}

		#region Help
		/// <summary>
		/// Draws a CAD image from the loaded CAD file.
		/// </summary>
		/// <param name="gr">A <see cref="System.Drawing.Graphics">Graphics</see> object used to draw the CAD entities.</param>
		#endregion Help
		public void DrawCADImage(Graphics gr)
		{						
			try
			{								
				Shift();
				RectangleF tmp = ImageRectangleF;															
				SetSizePictureBox(new Size((int)tmp.Width, (int)tmp.Height));	
				cadImage.Draw(gr, tmp);				
			}
			catch
			{
				return;
			}			
		}

		private void cadPictBox_MouseUp(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			detRMouseDown = false;
			cadPictBox.Cursor = Cursors.Default;
			cadPictBox.Invalidate();
		}

		private void cadPictBox_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			if(e.Button == MouseButtons.Right) 
			{
				MainForm.ActiveForm.Cursor = Cursors.Hand;
				cX = e.X;
				cY = e.Y;
				detRMouseDown = true;
			}
		}

		private void cadPictBox_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			cadPictBox.Focus();
			if(cadImage == null) return;
			if(detRMouseDown)
			{
				pos.X -= (cX - e.X);
				pos.Y -= (cY - e.Y);
				cX = e.X;
				cY = e.Y;				
				cadPictBox.Invalidate();
				SetPictureBoxPosition(Position);
			}
			PreviousPosition = new PointF(e.X, e.Y);
			DPoint vPt = GetRealPoint(e.X, e.Y);
			stBar.Panels[2].Text = vPt.X.ToString() + ApplicationConstants.sepstr2 + vPt.Y.ToString() + 
						  		   ApplicationConstants.sepstr2 + vPt.Z.ToString();			
		}

		#region Help
		/// <summary>
		/// Gets a three-dimensional point of the current CAD image from the specified screen point.
		/// </summary>
		/// <param name="x">An X coordinate of the screen point.</param>
		/// <param name="y">A Y coordinate of the screen point.</param>
		/// <returns>A three-dimensional point.</returns>
		#endregion Help
		public DPoint GetRealPoint(int x, int y)
		{			
			RectangleF tmpRect = ImageRectangleF;	
			return CADConst.GetRealPoint(x, y, this.cadImage, tmpRect);
		}	

		private void MoveToPosition(ref DPoint point, DPoint aPos, int direction)
		{  
			point.X = point.X + direction * aPos.X;
			point.Y = point.Y + direction * aPos.Y;
			point.Z = point.Z + direction * aPos.Z;
		}

		private void MainForm_Deactivate(object sender, System.EventArgs e)
		{
			detRMouseDown = false;
		}

		private void miExit_Click(object sender, System.EventArgs e)
		{
			Close();
		}

		private void miAbout_Click(object sender, System.EventArgs e)
		{
			aboutFrm.ShowDialog();
		}

		private void miZoom_Click(object sender, System.EventArgs e)
		{
			if(cadImage == null) 
				return;
			float i = ImageScale;
			switch((sender as MenuItem).Index)
			{
				case 0:
					i = 0.1f;
					break;
				case 1:
					i = 0.25f;
					break;
				case 2:
					i = 0.5f;
					break;
				case 3:
					i = 1.0f;
					break;
				case 4:
					i = 2.0f;
					break;
				case 5:
					i = 4.0f;
					break;
				case 6:
					i = 8.0f;
					break;
			}
			ResetScaling();
			this.PreviousPosition = new PointF(cadPictBox.ClientRectangle.Width / 2.0f, cadPictBox.ClientRectangle.Height / 2.0f);
			ImageScale = i;
			cadPictBox.Invalidate();
			SetPictureBoxPosition(Position);	
		}

		#region Help
		/// <summary>
		/// Sets the scale and position of the current CAD image to its original values.
		/// </summary>
		#endregion Help
		public void ResetScaling()
		{
			ImageScale = 1.0f;
			ImagePreviousScale = 1.0f;
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - VisibleAreaSize.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - VisibleAreaSize.Height) / 2.0f;
			cadPictBox.Invalidate();
		}		

		#region Help
		/// <summary>
		/// Resizes the CAD image to fit drawing's bounds.
		/// </summary>
		/// <remarks>This method is invoked when changing layout or working with a 3D Orbit tool.</remarks>
		#endregion Help
		public void DoResize()
		{				
			if(cadImage == null) 
				return;
			float wh = (float)(cadImage.AbsWidth / cadImage.AbsHeight); 
			float new_wh = (float)cadPictBox.ClientRectangle.Width / (float)cadPictBox.ClientRectangle.Height;
			if(cadImage is CADRasterImage)
				visibleArea = new SizeF((float)cadImage.AbsWidth, (float)cadImage.AbsHeight);
			else 
				visibleArea = cadPictBox.Size;
			if(new_wh > wh)
				visibleArea.Width = visibleArea.Height * wh;
			else 
			{
				if(new_wh < wh)
					visibleArea.Height = visibleArea.Width / wh;
				else
					visibleArea = cadPictBox.Size;
			}
			LeftImagePosition = (cadPictBox.ClientRectangle.Width - visibleArea.Width) / 2.0f;
			TopImagePosition = (cadPictBox.ClientRectangle.Height - visibleArea.Height) / 2.0f;				
			cadPictBox.Invalidate();
		}

		private void miSaveFile_Click(object sender, System.EventArgs e)
		{	
			string name = this.cadImage.Converter.Source;
			if(name != null)
			{
				name = Path.GetFileName(name).Replace(Path.GetExtension(name), string.Empty);			
				this.dlgSaveEnt.FileName = name;
			}
			if(dlgSaveEnt.ShowDialog() !=  DialogResult.OK) 
				return;									
			string fname = dlgSaveEnt.FileName;
			if(fname != null)							
			{
				if(dlgSaveEnt.FilterIndex == 1)
					this.xmlImporter.CreateXMLRecords(dlgSaveEnt.FileName, name + ApplicationConstants.xmlextstr2);			
				else
					this.txtImporter.CreateTXTRecords(name + ApplicationConstants.txtextstr);
			}			
		}
		
		private void MainForm_Load(object sender, System.EventArgs e)
		{			
			VisibleAreaSize = cadPictBox.Size;
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.FloatingLicRegister();
			}
			else	
			{
				Protection.Register();
			}
#else
			Protection.Register();
#endif
#endif
			#endregion
			#region demo
			#if Demo
				CADImport.CADImportForms.DemoForm d1 = new CADImport.CADImportForms.DemoForm();
				d1.ShowDialog();
			#endif
			#endregion demo
			CADConst.UseSHXFonts = false;
			CADConst.UseMultyTTFFonts = true;
			CADConst.UseTTFFonts = true;
		}

		private void miRegister_Click(object sender, System.EventArgs e)
		{
			#region protect
			#if protect
				regFrm.ShowDialog();
			#endif
			#endregion
		}

		private void MainForm_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			#region protect
#if protect
#if floatprotect
			if(Protection.LicenseType == LicenseType.Floating)
			{
				this.floatLicFrm.DisableFloating();
			}
			else
			{
				Protection.CloseApplication();
			}
#else
			Protection.CloseApplication();
#endif
#endif
			#endregion
		}

		private void miFloatLic_Click(object sender, System.EventArgs e)
		{
			#region protect
			#if floatprotect
				floatLicFrm.ShowDialog();
			#endif
			#endregion
		}
	}

	internal abstract class ApplicationConstants
	{		
		public static readonly string[] hatchStyle;
		public static readonly string sepstr;
		public static readonly string loadfilestr;
		public static readonly string msgBoxDebugCaption;
		public static readonly string notsupportedstr;
		public static readonly string notsupportedextstr;
		public static readonly string appnamestr;
		public static readonly string sepstr2;
		public static readonly string xmlextstr;
		public static readonly string xmlextstr2;
		public static readonly string sepstr3;
		public static readonly string txtextstr;

		static ApplicationConstants()
		{	
			hatchStyle = new string[8]{"BDiagonal", "Cross", "DiagCross", "FDiagonal",
									   "Horizontal", "PatternData", "Solid", "Vertical"};
			sepstr = " - ";
			sepstr2 = " : ";	
			loadfilestr = "Loading file...";
			msgBoxDebugCaption = "Debug application message";
			notsupportedstr = "not supported";
			notsupportedextstr = "Not supported in current version!";
			appnamestr = "CADImportNet Demo";
			xmlextstr = ".XML";
			xmlextstr2 = ".xml";
			sepstr3 = string.Empty + (char)13 + (char)10;
			txtextstr = ".txt";
		}

		public static string GetColor(Color val)
		{
			if(val.Equals(CADConst.clNone))
				return "[black/white]";
			if(val.Equals(CADConst.clByBlock))
				return "[ByBlock]";
			if(val.Equals(CADConst.clByLayer))
				return "[ByLayer]";				
			return val.ToString();			
		}
	}
}
