Imports System
Imports System.Drawing
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports System.IO
Imports CADImport
Imports CADImport.CADImportForms

Namespace CADImportNetDemos.CADMergeFilesDemo
    ''' <summary>
    ''' Represents a form in which a file to be merged can be selected and parameters of merging can be set.
    ''' </summary>
    Public Class MergeForm
        Inherits Form
        ' Events
        Public Event ResizeEvent As CADImport.FaceModule.CADImportEventHandler

        ' Methods
        ''' <summary>
        ''' Initializes a new instance of the <see cref="!:MergeFilesDemoApplication.MergeForm">MergeForm</see> class.
        ''' </summary>
        Public Sub New()
            Me.InitializeComponent()
            Me.InitExtParam()
        End Sub

        Private Sub btnDefaultVal_Click(ByVal sender As Object, ByVal e As EventArgs)
            Dim cntl As Control
            For Each cntl In Me.grbPosition.Controls
                If TypeOf cntl Is NumericUpDown Then
                    cntl.Text = ApplicationConstants.zerostr
                End If
            Next
            For Each cntl In Me.grbScale.Controls
                If TypeOf cntl Is NumericUpDown Then
                    cntl.Text = ApplicationConstants.hundredstr
                End If
            Next
            For Each cntl In Me.pnTools.Controls
                If TypeOf cntl Is NumericUpDown Then
                    cntl.Text = ApplicationConstants.zerostr
                End If
            Next
        End Sub

        Private Sub btnOK_Click(ByVal sender As Object, ByVal e As EventArgs)
            If (Me.filesToMerge.Count > 0) Then
                Dim i As Integer
                For i = 0 To Me.filesToMerge.Count - 1
                    Dim f1 As FileRecord = CType(Me.filesToMerge.Item(i), FileRecord)
                    Dim img1 As CADImage = Me.CreateCADImage(f1)
                    Me.Image.AddScaledDXF(img1, f1.FileName, f1.Position, f1.ImageScale, f1.Rotation)
                    Me.OnResize()
                Next i
                Me.Image.GetExtents()
            End If
            MyBase.Close()
        End Sub

        Private Sub btnOpenNext_Click(ByVal sender As Object, ByVal e As EventArgs)
            If (Me.dlgOpenFile.ShowDialog = DialogResult.OK) Then
                Dim pos As New DPoint(CDbl(Me.numX.Value), CDbl(Me.numY.Value), CDbl(Me.numZ.Value))
                Dim scPt As New DPoint((CDbl(Me.numXsc.Value) / 100.0), (CDbl(Me.numYsc.Value) / 100.0), (CDbl(Me.numZsc.Value) / 100.0))
                Dim f1 As New FileRecord(Me.dlgOpenFile.FileName, pos, scPt, CSng(Me.numRotation.Value))
                Me.filesToMerge.Add(f1)
                Me.cbFiles.Items.Add(String.Format("{0}. {1}", Me.cbFiles.Items.Count, Me.GetFileName(Me.dlgOpenFile.FileName)))
                Me.cbFiles.SelectedItem = Me.cbFiles.Items.Item((Me.cbFiles.Items.Count - 1))
            End If
        End Sub

        Private Sub cbFiles_SelectedValueChanged(ByVal sender As Object, ByVal e As EventArgs)
            Me.lbCnFiles.Text = Me.filesToMerge.Count.ToString
            Dim f1 As FileRecord = DirectCast(Me.filesToMerge.Item(Me.cbFiles.SelectedIndex), FileRecord)
            Me.numX.Value = CDec(f1.Position.X)
            Me.numY.Value = CDec(f1.Position.Y)
            Me.numZ.Value = CDec(f1.Position.Z)
            Me.numXsc.Value = CDec((f1.ImageScale.X * 100.0!))
            Me.numYsc.Value = CDec((f1.ImageScale.Y * 100.0!))
            Me.numZsc.Value = CDec((f1.ImageScale.Z * 100.0!))
            Me.numRotation.Value = CDec(f1.Rotation)
            Me.prevIndex = Me.cbFiles.SelectedIndex
        End Sub

        Private Function CreateCADImage(ByVal rec As FileRecord) As CADImage
            Dim img As CADImage = CADImage.CreateImageByExtension(rec.FileName)
            If (img Is Nothing) Then
                Return Nothing
            End If
            img.LoadFromFile(rec.FileName)
            img.UseDoubleBuffering = False
            img.BackgroundColor = Me.Image.BackgroundColor
            img.DefaultColor = Me.Image.DefaultColor
            img.DrawMode = Me.Image.DrawMode
            Return img
        End Function

        ''' <summary>
        ''' Cleans up any resources being used.
        ''' </summary>
        ''' <param name="disposing">A value indicating if both managed and unmanaged resources have to be released (<b>true</b>) or only unmanaged (<b>false</b>). 
        ''' </param>
        Protected Overloads Sub Dispose(ByVal disposing As Boolean)
            If (disposing AndAlso (Not Me.components Is Nothing)) Then
                Me.components.Dispose()
            End If
            MyBase.Dispose(disposing)
        End Sub

        Private Function GetFileName(ByVal path As String) As String
            Return System.IO.Path.GetFileName(path)
        End Function

        Private Sub InitExtParam()
            Me.filesToMerge = New ArrayList
            Me.prevIndex = -1
        End Sub

#Region " Windows Form Designer generated code "
        ''' <summary>
        ''' Required method for Designer support - do not modify
        ''' the contents of this method with the code editor.
        ''' </summary>
        Private Sub InitializeComponent()
            Me.pnTools = New Panel
            Me.btnOpenNext = New Button
            Me.cbFiles = New ComboBox
            Me.lbCnFiles = New Label
            Me.lbFiles = New Label
            Me.btnDefaultVal = New Button
            Me.lbRotation = New Label
            Me.numRotation = New NumericUpDown
            Me.grbScale = New GroupBox
            Me.lbPercent3 = New Label
            Me.lbPercent2 = New Label
            Me.lbScZ = New Label
            Me.lbScY = New Label
            Me.numXsc = New NumericUpDown
            Me.numYsc = New NumericUpDown
            Me.numZsc = New NumericUpDown
            Me.lbScX = New Label
            Me.lbPercent1 = New Label
            Me.grbPosition = New GroupBox
            Me.lbZ = New Label
            Me.lbY = New Label
            Me.numZ = New NumericUpDown
            Me.numY = New NumericUpDown
            Me.numX = New NumericUpDown
            Me.lbX = New Label
            Me.btnOK = New Button
            Me.dlgOpenFile = New OpenFileDialog
            Me.pnTools.SuspendLayout()
            Me.numRotation.BeginInit()
            Me.grbScale.SuspendLayout()
            Me.numXsc.BeginInit()
            Me.numYsc.BeginInit()
            Me.numZsc.BeginInit()
            Me.grbPosition.SuspendLayout()
            Me.numZ.BeginInit()
            Me.numY.BeginInit()
            Me.numX.BeginInit()
            MyBase.SuspendLayout()
            Me.pnTools.BorderStyle = BorderStyle.Fixed3D
            Me.pnTools.Controls.Add(Me.btnOpenNext)
            Me.pnTools.Controls.Add(Me.cbFiles)
            Me.pnTools.Controls.Add(Me.lbCnFiles)
            Me.pnTools.Controls.Add(Me.lbFiles)
            Me.pnTools.Controls.Add(Me.btnDefaultVal)
            Me.pnTools.Controls.Add(Me.lbRotation)
            Me.pnTools.Controls.Add(Me.numRotation)
            Me.pnTools.Controls.Add(Me.grbScale)
            Me.pnTools.Controls.Add(Me.grbPosition)
            Me.pnTools.Location = New Point(0, 0)
            Me.pnTools.Name = "pnTools"
            Me.pnTools.Size = New Size(&HF2, &HF6)
            Me.pnTools.TabIndex = 0
            Me.btnOpenNext.FlatStyle = FlatStyle.System
            Me.btnOpenNext.Location = New Point(&H8E, &HD5)
            Me.btnOpenNext.Name = "btnOpenNext"
            Me.btnOpenNext.Size = New Size(&H58, &H18)
            Me.btnOpenNext.TabIndex = 11
            Me.btnOpenNext.Text = "Open next file"
            AddHandler Me.btnOpenNext.Click, New System.EventHandler(AddressOf Me.btnOpenNext_Click)
            Me.cbFiles.Location = New Point(8, &HB8)
            Me.cbFiles.Name = "cbFiles"
            Me.cbFiles.Size = New Size(&HE0, &H15)
            Me.cbFiles.TabIndex = 10
            AddHandler Me.cbFiles.SelectedValueChanged, New System.EventHandler(AddressOf Me.cbFiles_SelectedValueChanged)
            Me.lbCnFiles.FlatStyle = FlatStyle.System
            Me.lbCnFiles.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbCnFiles.Location = New Point(200, &HA1)
            Me.lbCnFiles.Name = "lbCnFiles"
            Me.lbCnFiles.Size = New Size(&H19, &H10)
            Me.lbCnFiles.TabIndex = 9
            Me.lbCnFiles.Text = "0"
            Me.lbCnFiles.TextAlign = ContentAlignment.MiddleLeft
            Me.lbFiles.FlatStyle = FlatStyle.System
            Me.lbFiles.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbFiles.Location = New Point(8, 160)
            Me.lbFiles.Name = "lbFiles"
            Me.lbFiles.Size = New Size(&HB8, &H10)
            Me.lbFiles.TabIndex = 8
            Me.lbFiles.Text = "Number of files to be merged :"
            Me.btnDefaultVal.FlatStyle = FlatStyle.System
            Me.btnDefaultVal.Location = New Point(&H8E, &H7E)
            Me.btnDefaultVal.Name = "btnDefaultVal"
            Me.btnDefaultVal.Size = New Size(&H58, &H18)
            Me.btnDefaultVal.TabIndex = 7
            Me.btnDefaultVal.Text = "Default values"
            AddHandler Me.btnDefaultVal.Click, New System.EventHandler(AddressOf Me.btnDefaultVal_Click)
            Me.lbRotation.FlatStyle = FlatStyle.System
            Me.lbRotation.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbRotation.Location = New Point(0, &H7E)
            Me.lbRotation.Name = "lbRotation"
            Me.lbRotation.Size = New Size(&H40, &H17)
            Me.lbRotation.TabIndex = 6
            Me.lbRotation.Text = "Rotation :"
            Me.lbRotation.TextAlign = ContentAlignment.MiddleCenter
            Me.numRotation.Location = New Point(&H40, &H80)
            Dim tmp0 As Integer() = New Integer(4 - 1) {}
            tmp0(0) = &H186A0
            Me.numRotation.Maximum = New Decimal(tmp0)
            Dim tmp1 As Integer() = New Integer(4 - 1) {}
            tmp1(0) = &H186A0
            tmp1(3) = -2147483648
            Me.numRotation.Minimum = New Decimal(tmp1)
            Me.numRotation.Name = "numRotation"
            Me.numRotation.Size = New Size(&H38, 20)
            Me.numRotation.TabIndex = 5
            Me.grbScale.Controls.Add(Me.lbPercent3)
            Me.grbScale.Controls.Add(Me.lbPercent2)
            Me.grbScale.Controls.Add(Me.lbScZ)
            Me.grbScale.Controls.Add(Me.lbScY)
            Me.grbScale.Controls.Add(Me.numXsc)
            Me.grbScale.Controls.Add(Me.numYsc)
            Me.grbScale.Controls.Add(Me.numZsc)
            Me.grbScale.Controls.Add(Me.lbScX)
            Me.grbScale.Controls.Add(Me.lbPercent1)
            Me.grbScale.FlatStyle = FlatStyle.System
            Me.grbScale.Location = New Point(&H80, 8)
            Me.grbScale.Name = "grbScale"
            Me.grbScale.Size = New Size(&H68, &H68)
            Me.grbScale.TabIndex = 1
            Me.grbScale.TabStop = False
            Me.grbScale.Text = "Scale"
            Me.lbPercent3.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbPercent3.Location = New Point(80, &H51)
            Me.lbPercent3.Name = "lbPercent3"
            Me.lbPercent3.Size = New Size(&H10, &H10)
            Me.lbPercent3.TabIndex = 8
            Me.lbPercent3.Text = "%"
            Me.lbPercent3.TextAlign = ContentAlignment.MiddleCenter
            Me.lbPercent2.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbPercent2.Location = New Point(80, &H33)
            Me.lbPercent2.Name = "lbPercent2"
            Me.lbPercent2.Size = New Size(&H10, &H10)
            Me.lbPercent2.TabIndex = 7
            Me.lbPercent2.Text = "%"
            Me.lbPercent2.TextAlign = ContentAlignment.MiddleCenter
            Me.lbScZ.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbScZ.Location = New Point(8, 80)
            Me.lbScZ.Name = "lbScZ"
            Me.lbScZ.Size = New Size(&H18, &H10)
            Me.lbScZ.TabIndex = 6
            Me.lbScZ.Text = "Z :"
            Me.lbScZ.TextAlign = ContentAlignment.MiddleCenter
            Me.lbScY.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbScY.Location = New Point(8, &H33)
            Me.lbScY.Name = "lbScY"
            Me.lbScY.Size = New Size(&H18, &H10)
            Me.lbScY.TabIndex = 5
            Me.lbScY.Text = "Y :"
            Me.lbScY.TextAlign = ContentAlignment.MiddleCenter
            Me.numXsc.Location = New Point(&H20, 20)
            Dim tmp2 As Integer() = New Integer(4 - 1) {}
            tmp2(0) = &H3E8
            Me.numXsc.Maximum = New Decimal(tmp2)
            Me.numXsc.Name = "numXsc"
            Me.numXsc.Size = New Size(&H30, 20)
            Me.numXsc.TabIndex = 4
            Dim tmp3 As Integer() = New Integer(4 - 1) {}
            tmp3(0) = 100
            Me.numXsc.Value = New Decimal(tmp3)
            Me.numYsc.Location = New Point(&H20, &H31)
            Dim tmp4 As Integer() = New Integer(4 - 1) {}
            tmp4(0) = &H3E8
            Me.numYsc.Maximum = New Decimal(tmp4)
            Me.numYsc.Name = "numYsc"
            Me.numYsc.Size = New Size(&H30, 20)
            Me.numYsc.TabIndex = 3
            Dim tmp5 As Integer() = New Integer(4 - 1) {}
            tmp5(0) = 100
            Me.numYsc.Value = New Decimal(tmp5)
            Me.numZsc.Location = New Point(&H20, &H4E)
            Dim tmp6 As Integer() = New Integer(4 - 1) {}
            tmp6(0) = &H3E8
            Me.numZsc.Maximum = New Decimal(tmp6)
            Me.numZsc.Name = "numZsc"
            Me.numZsc.Size = New Size(&H30, 20)
            Me.numZsc.TabIndex = 2
            Dim tmp7 As Integer() = New Integer(4 - 1) {}
            tmp7(0) = 100
            Me.numZsc.Value = New Decimal(tmp7)
            Me.lbScX.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbScX.Location = New Point(8, &H16)
            Me.lbScX.Name = "lbScX"
            Me.lbScX.Size = New Size(&H18, &H10)
            Me.lbScX.TabIndex = 1
            Me.lbScX.Text = "X :"
            Me.lbScX.TextAlign = ContentAlignment.MiddleCenter
            Me.lbPercent1.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbPercent1.Location = New Point(80, &H17)
            Me.lbPercent1.Name = "lbPercent1"
            Me.lbPercent1.Size = New Size(&H10, &H10)
            Me.lbPercent1.TabIndex = 2
            Me.lbPercent1.Text = "%"
            Me.lbPercent1.TextAlign = ContentAlignment.MiddleCenter
            Me.grbPosition.Controls.Add(Me.lbZ)
            Me.grbPosition.Controls.Add(Me.lbY)
            Me.grbPosition.Controls.Add(Me.numZ)
            Me.grbPosition.Controls.Add(Me.numY)
            Me.grbPosition.Controls.Add(Me.numX)
            Me.grbPosition.Controls.Add(Me.lbX)
            Me.grbPosition.FlatStyle = FlatStyle.System
            Me.grbPosition.Location = New Point(8, 8)
            Me.grbPosition.Name = "grbPosition"
            Me.grbPosition.Size = New Size(&H70, &H68)
            Me.grbPosition.TabIndex = 0
            Me.grbPosition.TabStop = False
            Me.grbPosition.Text = "Position"
            Me.lbZ.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbZ.Location = New Point(8, 80)
            Me.lbZ.Name = "lbZ"
            Me.lbZ.Size = New Size(&H18, &H10)
            Me.lbZ.TabIndex = 6
            Me.lbZ.Text = "Z :"
            Me.lbZ.TextAlign = ContentAlignment.MiddleCenter
            Me.lbY.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbY.Location = New Point(8, &H33)
            Me.lbY.Name = "lbY"
            Me.lbY.Size = New Size(&H18, &H10)
            Me.lbY.TabIndex = 5
            Me.lbY.Text = "Y :"
            Me.lbY.TextAlign = ContentAlignment.MiddleCenter
            Me.numZ.Location = New Point(&H20, &H4E)
            Dim tmp8 As Integer() = New Integer(4 - 1) {}
            tmp8(0) = &H186A0
            Me.numZ.Maximum = New Decimal(tmp8)
            Dim tmp9 As Integer() = New Integer(4 - 1) {}
            tmp9(0) = &H186A0
            tmp9(3) = -2147483648
            Me.numZ.Minimum = New Decimal(tmp9)
            Me.numZ.Name = "numZ"
            Me.numZ.Size = New Size(&H48, 20)
            Me.numZ.TabIndex = 4
            Me.numY.Location = New Point(&H20, &H31)
            Dim tmp10 As Integer() = New Integer(4 - 1) {}
            tmp10(0) = &H186A0
            Me.numY.Maximum = New Decimal(tmp10)
            Dim tmp11 As Integer() = New Integer(4 - 1) {}
            tmp11(0) = &H186A0
            tmp11(3) = -2147483648
            Me.numY.Minimum = New Decimal(tmp11)
            Me.numY.Name = "numY"
            Me.numY.Size = New Size(&H48, 20)
            Me.numY.TabIndex = 3
            Me.numX.Location = New Point(&H20, 20)
            Dim tmp12 As Integer() = New Integer(4 - 1) {}
            tmp12(0) = &H186A0
            Me.numX.Maximum = New Decimal(tmp12)
            Dim tmp13 As Integer() = New Integer(4 - 1) {}
            tmp13(0) = &H186A0
            tmp13(3) = -2147483648
            Me.numX.Minimum = New Decimal(tmp13)
            Me.numX.Name = "numX"
            Me.numX.Size = New Size(&H48, 20)
            Me.numX.TabIndex = 2
            Me.lbX.Font = New Font("Microsoft Sans Serif", 9.75!, FontStyle.Regular, GraphicsUnit.Point, &HCC)
            Me.lbX.Location = New Point(8, &H16)
            Me.lbX.Name = "lbX"
            Me.lbX.Size = New Size(&H18, &H10)
            Me.lbX.TabIndex = 1
            Me.lbX.Text = "X :"
            Me.lbX.TextAlign = ContentAlignment.MiddleCenter
            Me.btnOK.FlatStyle = FlatStyle.System
            Me.btnOK.Location = New Point(&H8E, 250)
            Me.btnOK.Name = "btnOK"
            Me.btnOK.Size = New Size(&H58, &H18)
            Me.btnOK.TabIndex = 3
            Me.btnOK.Text = "OK"
            AddHandler Me.btnOK.Click, New System.EventHandler(AddressOf Me.btnOK_Click)
            Me.dlgOpenFile.Filter = "AutoCAD files(*.dwg;*.dxf)|*.dwg;*.dxf"
            Me.AutoScaleBaseSize = New Size(5, 13)
            MyBase.ClientSize = New Size(&HF2, &H115)
            MyBase.Controls.Add(Me.btnOK)
            MyBase.Controls.Add(Me.pnTools)
            MyBase.FormBorderStyle = FormBorderStyle.FixedSingle
            MyBase.MaximizeBox = False
            MyBase.MinimizeBox = False
            MyBase.Name = "MergeForm"
            MyBase.StartPosition = FormStartPosition.CenterParent
            Me.Text = "Merge Scaled Files"
            Me.pnTools.ResumeLayout(False)
            Me.numRotation.EndInit()
            Me.grbScale.ResumeLayout(False)
            Me.numXsc.EndInit()
            Me.numYsc.EndInit()
            Me.numZsc.EndInit()
            Me.grbPosition.ResumeLayout(False)
            Me.numZ.EndInit()
            Me.numY.EndInit()
            Me.numX.EndInit()
            MyBase.ResumeLayout(False)
        End Sub
#End Region

        Public Overloads Sub OnResize()
            RaiseEvent ResizeEvent()
        End Sub


        ' Properties
        Public Property Image() As CADImage
            Get
                Return Me.img
            End Get
            Set(ByVal value As CADImage)
                Me.img = value
            End Set
        End Property


        ' Fields
        Private btnDefaultVal As Button
        Private btnOK As Button
        Private btnOpenNext As Button
        Private cbFiles As ComboBox
        Private components As Container = Nothing
        Private dlgOpenFile As OpenFileDialog
        Private filesToMerge As ArrayList
        Private grbPosition As GroupBox
        Private grbScale As GroupBox
        Private img As CADImage
        Private lbCnFiles As Label
        Private lbFiles As Label
        Private lbPercent1 As Label
        Private lbPercent2 As Label
        Private lbPercent3 As Label
        Private lbRotation As Label
        Private lbScX As Label
        Private lbScY As Label
        Private lbScZ As Label
        Private lbX As Label
        Private lbY As Label
        Private lbZ As Label
        Private numRotation As NumericUpDown
        Private numX As NumericUpDown
        Private numXsc As NumericUpDown
        Private numY As NumericUpDown
        Private numYsc As NumericUpDown
        Private numZ As NumericUpDown
        Private numZsc As NumericUpDown
        Private pnTools As Panel
        Private prevIndex As Integer
    End Class
End Namespace